


/**************************************************************************
 * Projet E C 2 O C
 * -------------------------------------------------------------------------
 * module   : ec.yacc
 *
 * date     : 20/1/93
 * auteurs  : Pascal Raymond
 *--------------------------------------------------------------------------
 * nature   : Analyseur syntaxique du code 'ec' decrit en YACC
 *--------------------------------------------------------------------------
 * Modifs   :
 *			4/9/94 : version ec "simplifiee" 
 *		La presence de l'horloge lors de la declaration de
 *	constante, parametre ou variable locale est OPTIONNELLE,
 *	par defaut, l'horloge est l'horloge des flots est l'horloge de BASE.
 *	Cela permet d'avoir du code "Lustre plat"	 re-entrant.
 *			14/3/97 : toutes les listes deviennent recursives a gauche
 * dans la grammaire. 
 *
 *************************************************************************/

/* --------------------------------------------------------------------
  Entete du fichier c 
----------------------------------------------------------------------*/
%{
//#include <alloca.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "utiles.h"
#include "net.h"
#include "ecconst.h"

/*---- Quelques declarations ------------------------------*/
void yyerror (const char *s ); 
int yylex (); 

/*----- VARIABLES GLOBALES ---------------------------------*/
static SymbolTable* Symbols;
static Net* net;
static int net_defined;
static FILE* infile;

/*----- UTILITAIRES ----------------------------------------*/
static int AddSymbol(const char* nme){
	return Symbols->add(nme);
}
static IntList* AddIntList(IntList* l1, IntList* l2){
	l1->concat(l2); return l1;
}
static IntList* GetIntList(int i){
	return new IntList(i);
}
static ExpList* AddExpList(ExpList* l1, ExpList* l2){
	l1->concat(l2); return l1;
}
static ExpList* GetExpList(Exp* i){
	return new ExpList(i);
}
static void DefineNodeName(int name){
	if (net_defined) {
		fprintf(stderr, "EcParse: ec program must contain a single node\n");
		exit(1);
	} else {
		net_defined = 1;
		net->def_name(name);
	}
}
static void AddUserConsts(IntList* names, int type, int clock){
	IntIterator next_name(names);
	int name;
	while((name = next_name())){
		net->add_usr_const(name, net->define_type(type), clock);
	}
}
static void AddExternType(int name){
	net->add_usr_type(name);
}
static void AddEnumType(int name, IntList* consts){
	net->add_usr_type(name, consts);
}

static void AddUserFunction(int name, int usf, IntList* in_params, IntList* out_params){
	/* Les listes de params alternent des indice de noms et de types
		on doit en extraire la liste des type pour construire la 
		structure EcType correspondante :
	*/ 
	IntList* in_types, *out_types;
	in_types = out_types = NULL;
	IntList* params;

	params = in_params;
	while(params){
		params = params->tail();				/* On ignore le nom */
		append_list(in_types, params->head());	/* pour accedeer au type */
		params = params->tail();
	}
	params = out_params;
	while(params){
		params = params->tail();
		append_list(out_types, params->head());
		params = params->tail();
	}

	net->add_usr_func(name, usf, 
		net->define_type(in_types),
		net->define_type(out_types),
		in_params,
		out_params);
}

static void AddUserNode(int name, IntList* in_params, IntList* out_params){
	/* Les listes de params alternent des indice de noms et de types
		on doit en extraire la liste des type pour construire la 
		structure EcType correspondante :
	*/ 
	IntList* in_types, *out_types;
	in_types = out_types = NULL;
	IntList* params;

	params = in_params;
	while(params){
		params = params->tail();				/* On ignore le nom */
		append_list(in_types, params->head());	/* pour accedeer au type */
		params = params->tail();
	}
	params = out_params;
	while(params){
		params = params->tail();
		append_list(out_types, params->head());
		params = params->tail();
	}
	//On force les node comme procedures unsafe ...
	Warning("basic support of extern node (treated as unsafe procedure)");
	net->add_usr_unsafe_proc(name, 
		net->define_type(in_types),
		net->define_type(out_types),
		in_params,
		out_params);
}

static int BaseClock(){
	return net->base_clock();
}
static void AddInputs(IntList* names, int type, int clock){
	IntIterator next_name(names);
	int name;
	while((name = next_name())){
		net->add_input(name, net->define_type(type), clock);
	}
}
static void AddOutputs(IntList* names, int type, int clock){
	IntIterator next_name(names);
	int name;
	while((name = next_name())){
		net->add_output(name, net->define_type(type), clock);
	}
}
static void AddLocals(IntList* names, int type, int clock){
	IntIterator next_name(names);
	int name;
	while((name = next_name())){
		net->add_local(name, net->define_type(type), clock);
	}
}
static void AddAssertion(Exp* assertion){
	net->add_assertion(assertion);
}
static void AddEquation(int var, Exp* def){
	net->define_variable(var,def);
}
static void AddTupleEquation(IntList* vars, Exp* def){
	net->define_tuple(vars,def);
}
static Exp* GetIdentExp(int ident){
	IdentInfo* info;

	info = net->get_ident_info(ident);
	if(!info){
		fprintf(stderr,
			"ecnet::GetIdentInfo : undeclared ident '%s'\n",
			(*Symbols)[ident]);
		exit(1);
	}
	return info->ec_ref();
}
static Exp* GetBoolConst(int string){
	return net->get_bool_const(string);
}
static Exp* GetIntConst(int string){
	return net->get_int_const(string);
}
static Exp* GetRealConst(int string){
	return net->get_real_const(string);
}
static Exp* UExp(ExpNature nature, Exp* operand){
	return net->get_exp(nature, operand);
}
static Exp* BExp(ExpNature nature, Exp* op1, Exp* op2){
	return net->get_exp(nature, op1, op2);
}
static Exp* TExp(ExpNature nature, Exp* op1, Exp* op2, Exp* op3){
	return net->get_exp(nature, op1, op2, op3);
}
static Exp* NExp(ExpNature nature, ExpList* ops){
	return net->get_exp(nature, ops);
}
/* Construction d'un call : on oblige a 'tupler' l'eventuelle liste d'entrees*/
static Exp* GetCallExp(int func, Exp* op){
	Exp* res;

	res = net->get_exp(CALL_n, op, func);
	return res;
}

static int int2real_undef = 1;
Exp* I2RExp(Exp* operand){
   /* construit la fonction externe `real` */
	if(int2real_undef){ 
   	AddUserFunction(
      	AddSymbol("int2real"), _SAFE_EXTERN,
      	AddIntList(GetIntList(AddSymbol("x")), GetIntList(AddSymbol("int"))),
      	AddIntList(GetIntList(AddSymbol("y")), GetIntList(AddSymbol("real")))
   	);
		int2real_undef = 0;
	}
   return GetCallExp(AddSymbol("int2real"), operand);
}

static int real2int_undef = 1;
Exp* R2IExp(Exp* operand){
   /* construit la fonction externe `int` */
	if(real2int_undef){
   	AddUserFunction(
      	AddSymbol("real2int"), _SAFE_EXTERN,
      	AddIntList(GetIntList(AddSymbol("x")), GetIntList(AddSymbol("real"))),
      	AddIntList(GetIntList(AddSymbol("y")), GetIntList(AddSymbol("int")))
   	);
		real2int_undef = 0;
	}
   return GetCallExp(AddSymbol("real2int"), operand);
}

%}


/* --------------------------------------------------------------------
  Point d'entree de l'analyse 
----------------------------------------------------------------------*/

%start EcFile

/* --------------------------------------------------------------------
  Type de yylval (resultat de l'analyse)
----------------------------------------------------------------------*/
%union 
{
	char StringVal[MAXIDF];
	int Index;
	IntList* IndexList;
	Exp* pExp;
	ExpList* pExpList;
}

/* --------------------------------------------------------------------
  Terminaux
----------------------------------------------------------------------*/

%token XAnd 
%token XArrow 
%token XAssert 
%token <StringVal> XBool 
%token XConst
%token XCurrent
%token XDiv
%token XElse
%token XFunction
%token XExtern
%token XUnsafe
%token <StringVal> XFalse
%token XGTE
%token XGT
%token <StringVal> XIConst 
%token <StringVal> XIdf 
%token XIf
%token <StringVal> XInt
%token XImpl
%token XLTE
%token XLT
%token XLet
%token XMod
%token XNE
%token XNode
%token XNor
%token XNot
%token XOr
%token XPre
%token <StringVal> XRConst 
%token <StringVal> XReal
%token XReturns
%token XTel
%token XType 
%token XThen
%token <StringVal> XTrue
%token XVar
%token XWhen
%token XXor

/* --------------------------------------------------------------------
  Non Terminaux
----------------------------------------------------------------------*/


%type <Index> Type
%type <Index> PredefinedType
%type <Index> UserType
%type <Index> Clock
%type <IndexList> IdentList
%type <Index> Ident
%type <IndexList> ParamList
%type <IndexList> FunParams
%type <IndexList> Parameter
%type <pExp> Expression
%type <pExp> CallExpression
%type <pExpList> ExpList2
%type <pExpList> ExpressionList
%type <pExp> Constant


/* --------------------------------------------------------------------
  Priorites
----------------------------------------------------------------------*/

%left XElse
%left XArrow
%right XImpl
%left XOr
%left XXor
%left XAnd
%left '=' XNE
%left '<' XLTE XGTE '>'
%left '+' '-'
%nonassoc XNot
%left '*' '/' '%' XMod XDiv
%left XPower
%left XWhen
%nonassoc XInt XReal
%nonassoc XUMinus XPre XCurrent '#' '&' XNor

/* --------------------------------------------------------------------
  Grammaire 
----------------------------------------------------------------------*/
%%

EcFile:	/* empty */
	|	DeclList
	;

/* declarations */

DeclList:	OneDecl
	|	DeclList OneDecl
	;

OneDecl: 	ConstDecl 
	|	      TypeDecl
	|	      FuncDecl
	|	      NodeDecl
	;

/* constantes */

ConstDecl:	XConst UsrConstDeclList '.'
	|	      XConst UsrConstDeclList ';'
	;

UsrConstDeclList:	UsrConstDecl
	|	UsrConstDeclList ';' UsrConstDecl
	;

UsrConstDecl:	'(' IdentList ':' Type ')' XWhen Clock
			{AddUserConsts($2,$4,$7);}
	|		IdentList ':' Type
			{AddUserConsts($1,$3,BaseClock());}
	; 

Type:	PredefinedType
			{$$ = $1;}
	|	UserType
			{$$ = $1;}
	;

PredefinedType: XBool 
			{$$ = AddSymbol($1);}
	|	XInt 
			{$$ = AddSymbol($1);}
	|	XReal
			{$$ = AddSymbol($1);}
	;

UserType:	Ident
			{$$ = $1;}
	;

Clock:		Ident
			{$$ = $1;}
	|	XTrue
			{$$ = AddSymbol($1);}
	;

/* types */

TypeDecl:	XType UsrTypeDeclList ';'
	|	      XType UsrTypeDeclList '.'
	;

UsrTypeDeclList :	UsrTypeDecl
	|	            UsrTypeDeclList ',' UsrTypeDecl
	;

UsrTypeDecl :	ExternTypeDecl
	|				EnumTypeDecl
	;

ExternTypeDecl : Ident
			{AddExternType($1);}
	;

EnumTypeDecl : Ident '=' '('  IdentList ')'
			{AddEnumType($1, $4);}
	;

Ident :	XIdf
				{$$ = AddSymbol($1);}
	;

/* fonctions */

FuncDecl: XFunction Ident '(' FunParams ')'  XReturns '(' FunParams ')' EndDecl 
			{ AddUserFunction($2,_SAFE_EXTERN,$4,$8);}
	|		XUnsafe XFunction Ident '(' FunParams ')'  XReturns '(' FunParams ')' EndDecl 
			{ AddUserFunction($3,_UNSAFE_EXTERN,$5,$9);}
	|		XExtern XNode Ident '(' FunParams ')'  XReturns '(' FunParams ')' EndDecl 
			{ AddUserNode($3,$5,$9);}
	;

FunParams : ParamList OptSCol
            { $$ = $1; }
      ;

ParamList:	Parameter
			{$$ = $1;}
	|
		      ParamList ';' Parameter
			{$$ = AddIntList($1,$3);}
	;

/* On construit une liste heterogene ou alterne les noms et les types 
	Cette liste doit etre "cassee" dans AddUserFunction 
*/
Parameter:	Ident ':' Type
			{ $$ = AddIntList(GetIntList($1),GetIntList($3));}
	;

/* noeuds */

OptSCol :
		/* empty */
	|	';' /* semicol */
        ;


EndDecl :	
		/* empty */
	|	';' /* semicol */
	|	'.' 
  	| 	',' 
	;

NodeDecl:	XNode NodeHeader LocalDecls Equations EndDecl
	;

/* en-tete de noeud */

NodeHeader:	Ident '(' InputDeclList OptSCol ')' XReturns '(' OutputDeclList OptSCol ')' ';' 
			{DefineNodeName($1);}
	;

InputDeclList:	OneInputDecl
	|	InputDeclList ';' OneInputDecl
	;

OneInputDecl:	'(' IdentList ':' Type ')' XWhen Clock
			{AddInputs($2,$4,$7);}
	|            IdentList ':' Type 
			{AddInputs($1,$3,BaseClock());}
	; 

OutputDeclList:	OneOutputDecl
	|	OutputDeclList ';' OneOutputDecl
	;

OneOutputDecl:	'(' IdentList ':' Type ')' XWhen Clock
			{AddOutputs($2,$4,$7);}
	|            IdentList ':' Type 
			{AddOutputs($1,$3,BaseClock());}
	; 

/* variables locales */

LocalDecls:	/* empty */
	| 	XVar LocalDeclList ';'
	;

LocalDeclList:	OneLocalDecl
	|	LocalDeclList ';' OneLocalDecl
	;

OneLocalDecl:	'(' IdentList ':' Type ')' XWhen Clock
			{AddLocals($2,$4,$7);}
	| IdentList ':' Type 
			{AddLocals($1,$3,BaseClock());}
	; 

/* Corps d'un noeud */

Equations: 	XLet XTel
	|	XLet EquationList XTel
	;

/* Les Equations */

EquationList:	Equation
	|	EquationList Equation
	;

Equation:	XAssert Expression ';'
			{ AddAssertion($2);}
	|	Ident '=' Expression ';'
			{ AddEquation($1,$3);}
	|	'(' IdentList ')' '=' Expression ';'
			{ AddTupleEquation($2,$5);}
	;

IdentList : Ident
			{$$ = GetIntList($1);}
	|
				IdentList ',' Ident
			{$$ = AddIntList($1,GetIntList($3));}
	;

/* Les expressions */

Expression: 	'(' Expression ')'
			{$$ = $2;} 
	|	Constant
			{$$ = $1;}
	|	Ident 
			{$$ = GetIdentExp($1);} 
	|	XNot Expression
			{$$ = UExp(NOT_n,$2);}
	|	XInt Expression
			{$$ = R2IExp($2);}
	|	XReal Expression
			{$$ = I2RExp($2);}
	|	'-' Expression %prec XUMinus
			{$$ = UExp(UMINUS_n,$2);}  
	|	XPre Expression
			{$$ = UExp(PRE_n,$2);}  
	|	XCurrent Expression
			{$$ = UExp(CURRENT_n,$2);}
	|  Expression XArrow Expression
			{$$ = BExp(ARROW_n,$1,$3);}
	|  Expression XWhen Expression
			{$$ = BExp(WHEN_n,$1,$3);}
	|  XIf Expression XThen Expression XElse Expression
			{$$ = TExp (IF_n,$2,$4,$6);}
	|  Expression XXor Expression
			{$$ = BExp (XOR_n,$1,$3);}
	|  Expression XOr Expression
			{$$ = BExp (OR_n,$1,$3);}
	|  Expression XAnd Expression
			{$$ = BExp (AND_n,$1,$3);}
	|  Expression XImpl Expression
			{$$ = BExp (IMPL_n,$1,$3);}
	|  Expression '=' Expression
			{$$ = BExp (EQUAL_n,$1,$3);}
	|  Expression XNE Expression
			{$$ = BExp (NEQ_n,$1,$3);}
	|  Expression XLT Expression
			{$$ = BExp (LT_n,$1,$3);}
	|  Expression XLTE Expression
			{$$ = BExp (LTE_n,$1,$3);}
	|  Expression XGTE Expression
			{$$ = BExp (GTE_n,$1,$3);}
	|  Expression XGT Expression
			{$$ = BExp (GT_n,$1,$3);}
	|  Expression '*' Expression
			{$$ = BExp(TIMES_n,$1,$3);}
	|  Expression '/' Expression
			{$$ = BExp(SLASH_n,$1,$3);}
	|  Expression '%' Expression
			{$$ = BExp(MOD_n,$1,$3);}
	|  Expression XMod Expression
			{$$ = BExp(MOD_n,$1,$3);}
	|  Expression XDiv Expression
			{$$ = BExp(DIV_n,$1,$3);}
	|  Expression '+' Expression
			{$$ = BExp(PLUS_n,$1,$3);}
	|  Expression '-' Expression
			{$$ = BExp(MINUS_n,$1,$3);}  
	|  Expression XPower Expression
			{$$ = BExp(POWER_n,$1,$3);}  
	| 	CallExpression 
			{$$ = $1;}  
	|  '#' '(' ExpList2 ')'
			{$$ = NExp(DIESE_n,$3);}
	|  '&' '(' ExpList2 ')'
			{$$ = NExp(EXACTONE_n,$3);}
	|  XNor '(' ExpressionList ')'
			{$$ = NExp(NOR_n, $3);}
	|  '(' ExpList2 ')'
			{$$ = NExp(TUPLE_n, $2);}
	;

CallExpression: Ident '(' Expression ')'
		{
			//ICI 22/10/99
         $$ = GetCallExp($1,$3);
         //$$ = GetCallExp($1,NExp(TUPLE_n,GetExpList($3)));
      }
	|	Ident '(' ExpList2 ')'
			{$$ = GetCallExp($1,NExp(TUPLE_n,$3));}
	;

ExpList2: ExpressionList ',' Expression 
			{$$ = AddExpList($1, GetExpList($3));}
	;

ExpressionList:	Expression
			{$$ = GetExpList($1);}
	|	ExpList2 
			{$$ = $1;}
	;

Constant:	XTrue
			{$$ = GetBoolConst(AddSymbol($1));}
	|	XFalse
			{$$ = GetBoolConst(AddSymbol($1));}
	| 	XIConst 
			{$$ = GetIntConst(AddSymbol($1));}
	|	XRConst
			{$$ = GetRealConst(AddSymbol($1));}
	;


%%

typedef struct {
   const char *key ;
   int  token ;
   int  return_key; /* indique si yylex doit retourner la chaine */
                    /* en plus du token */
} keytoken;


#define NBKeywords 29

#define NBSpecKeywords 6

/* -------------------------------------------------------------------------
   Table des mots cle.
	'Keywords' sont ceux qui commencent par une lettre
	'SpecKeywords' sont ceux qui commencent par un separateur
------------------------------------------------------------------------- */

keytoken Keywords [NBKeywords] = {
	{"and", XAnd , 0},
 	{"assert", XAssert , 0},
	{"bool", XBool , 1},
 	{"const", XConst, 0},
 	{"current", XCurrent, 0},
 	{"div", XDiv, 0}, 
 	{"else", XElse, 0},
 	{"function", XFunction, 0},
 	{"false", XFalse, 1},
 	{"if", XIf, 0},
 	{"int", XInt, 1},
	{"let",  XLet, 0},
 	{"mod", XMod, 0},
 	{"node", XNode, 0},
	{"not", XNot, 0},
 	{"or", XOr, 0},
 	{"nor", XNor, 0},
 	{"pre", XPre, 0},
 	{"real", XReal, 1},
 	{"returns", XReturns, 0},
 	{"tel", XTel, 0},
	{"type", XType , 0},
 	{"then", XThen, 0},
 	{"true", XTrue, 1},
 	{"var", XVar, 0},
 	{"when", XWhen, 0},
	{"xor", XXor, 0},
 	{"extern", XExtern, 0},
 	{"unsafe", XUnsafe, 0},
};

keytoken SpecKeywords [NBSpecKeywords] = {
 	{"->", XArrow , 0},
 	{">=", XGTE, 0},
 	{"=>", XImpl, 0},
 	{"<=", XLTE, 0},
 	{"<>", XNE, 0},
 	{"**", XPower, 0}
};


/* -------------------------------------------------------------------------- 
   Variables et constantes necessaires a l'anlyse lexicale
-------------------------------------------------------------------------- */

//extern FILE *infile ;	/* defini par l'outil et son main */

static int yyline;	/* numero de ligne courant dans le fichier
			d'entree (necessaire pour le reperage des
			erreurs de syntaxe detectees par YACC */


/* -------------------------------------------------------------------------
   Fonction : getcar
	Fournit a yylex les caractere du fichier 'infile'
	Gere la variable 'yyline'
------------------------------------------------------------------------- */
static int vide1[1000];
static int carcour;
static int vide2[1000];

int cpt = 0;

int GetChar(int* pc) {
	int c ;
	c = fgetc(infile);	
//putchar(c);
cpt++;	
if(c == 0)
printf("ICI (car %d)\n", cpt);
	if(c == '\n') yyline++;
	*pc = c; 
	return c;
}


/* -------------------------------------------------------------------------
   Fonction : yyerror
   Fonctionnalite :
	Impression d'un message d'erreur sur le flot std de sortie
------------------------------------------------------------------------- */
void yyerror (const char *s ) {
	printf("%s     - at line %d\n", s, yyline) ;
} 

int yylexInit(){
	yyline = 1;
	return ((GetChar(&carcour)) != EOF);
}


/* -------------------------------------------------------------------------
   Fonction : issep1(int c)
	Renvoie 1 si 'c' est un separateur simple (i.e. il n'est
	pas prefixe d'un mot cle), sinon 0. 
------------------------------------------------------------------------- */
inline int issep1(int c) {
	return (
		(c == '.')	|| 
		(c == ';')	||
		(c == ')')	||
		(c == ':')	||
		(c == ',')	||
		(c == '/')	||
		(c == '%')	||
		(c == '+')	||
		(c == '#')  ||
		(c == '&')
	);
}

/* -------------------------------------------------------------------------
   Fonction : issep2(int c)
	Renvoie 1 si 'c' est un separateur qui est 
	aussi prefixe d'un mot cle, sinon 0. 
------------------------------------------------------------------------- */
inline int issep2(int c) { 
	return (
		(c == '=')	|| 
		(c == '(')	||
		(c == '-')	||
		(c == '<')	||
		(c == '>')	||
		(c == '*')
	);
}

/* -------------------------------------------------------------------------
   Fonction : islettre(int c)
	Renvoie 1 si 'c' est un caractere acceptable dans 
	un identificateur (alpha, digit, ou '_'), sinon 0. 
------------------------------------------------------------------------- */
inline int islettre(int c) {
	return (isalpha(c) || isdigit(c) || (c == '_'));
}


/* -------------------------------------------------------------------------
   Fonction : yylex

	Analyse lexicale des fichiers au format Argos.
   	Resultat  : un token
------------------------------------------------------------------------- */

char buffer[1024];

int yylex () {

char* sptr;
int result, i, isreal;

yylexInitState:

	/* On saute les blancs */
	while(isspace(carcour)) GetChar(&carcour);
	if(carcour == EOF) return carcour;
	else if(isdigit(carcour)) goto yylexDigitState;
	else if(islettre(carcour)) goto yylexIdentState; /* ne peut pas etre digit */
	else if(issep1(carcour)){
		result = carcour;
		GetChar(&carcour);
		return result;
	}
	else if(issep2(carcour)){
		buffer[0] = (char)(result = carcour);
		buffer[1] = (char)(GetChar(&carcour));
		buffer[2] = (char)0;
		/* Commentaire ? */	
		if(!strcmp(buffer,"(*")) goto yylexCommentState1;
		if(!strcmp(buffer,"--")) goto yylexCommentState2;
		/* Mot cle special ? */
		for(i=0;i<NBSpecKeywords;i++){
			if(!strcmp(buffer,SpecKeywords[i].key)){
				GetChar(&carcour);
				return SpecKeywords[i].token;
			}
		}
		/* On rend result, et on n'avance pas ! */
		return result;
	}
	else { /* caractere inconnu ! */
		printf("ec2oc lex error : unknown char '%d' (%c), line '%d'\n",
			carcour, (char)carcour,  yyline);
		exit(1);
	}

yylexDigitState:
	/* carcour pointe sur un digit */
	isreal = 0;
	sptr = &buffer[0];
	do {
		*(sptr++) = (char)carcour;
		GetChar(&carcour);	
	} while (isdigit(carcour));
	if(carcour == '.'){
		isreal = 1;
		do {
			*(sptr++) = (char)carcour;
			GetChar(&carcour);	
		} while (isdigit(carcour));
	} 
	if((carcour == 'e')||(carcour == 'E')){
		isreal = 1;
		*(sptr++) = (char)carcour;
		GetChar(&carcour);	
		/* Exposnat signe ? */	
		if((carcour == '+')||(carcour == '-')){
			*(sptr++) = (char)carcour;
			GetChar(&carcour);	
		}
		/* On attent obligatoirement un digit */
		if(isdigit(carcour)){
			do {
				*(sptr++) = (char)carcour;
				GetChar(&carcour);	
			} while (isdigit(carcour));
		} else {
			printf("ec2oc lex error : bad real syntax, line '%d'\n", yyline);
			exit(1);
		}
	}
	/* On sort de l'analyse : est-ce un reel ? */
	*sptr = '\0';
	if(isreal){
		strcpy(yylval.StringVal,buffer);
		return (XRConst);
	} else {
		strcpy(yylval.StringVal,buffer);
		return (XIConst);
	}

yylexIdentState:
	/* carcour pointe sur un caractere alpha */
	/* Remplisage de buffer : */
	sptr = &buffer[0];
	do {
		*(sptr++) = (char)carcour;
		GetChar(&carcour);
	} while (islettre(carcour));
	*sptr = '\0';
	/* Mot cle ? */
	for(i=0;i<NBKeywords;i++){
		if(!strcmp(buffer,Keywords[i].key)){
			if(Keywords[i].return_key) strcpy(yylval.StringVal,buffer);
			return Keywords[i].token;
		}
	}
	/* Identificateur */
	strcpy(yylval.StringVal,buffer);
	return (XIdf);

yylexCommentState1: 
	/* Commentaire debute par '(*' */
	do {
		result = carcour;
		GetChar(&carcour);
	} while( 
		((result != '*') || (carcour != ')'))
			 && 
		(carcour != EOF));		
	GetChar(&carcour);
	goto yylexInitState;

yylexCommentState2: 
	/* Commentaire debute par "--" */
	do {
		GetChar(&carcour) ;
	} while ((carcour != '\n') && (carcour != EOF));
	GetChar(&carcour);
	goto yylexInitState;


} /* fin de yylex */

/*----- PROCEDURE PRINCIPALE -------------------------------*/

int yyparse();

int EcParse(
	Net* n,          // Where to put the result
	SymbolTable* s,  // The symbol table to use
	const char* f          // The file to parse
){
	int res;
	if(!(infile = fopen(f, "r"))){
		fprintf(stderr, "EcParse : Can't open file '%s'\n", f);
		return 1;
	}
	Symbols = s;
	net = n;
	net_defined = 0;
	yylexInit();
	res = yyparse();

	if(!res && !net_defined) {
		fprintf(stderr, "EcParse: ec file must contain one node\n");
		return 1;
	}
	return res;
}

void EcLexTest(
	SymbolTable* s,  // The symbol table to use
	const char* f          // The file to parse
){
	if(!(infile = fopen(f, "r"))){
		fprintf(stderr, "EcLexTest : Can't open file '%s'\n", f);
		return;
	}
	Symbols = s;
	yylexInit();
	int lexeme;
	while((lexeme = yylex()) != EOF){
		if(issep1(lexeme) || issep2(lexeme)){
			printf("%c\n", (char)lexeme);
		} else if(lexeme == XRConst){
			printf("%s (*XRConst*)\n", yylval.StringVal);
		} else if(lexeme == XIConst){
			printf("%s (*XIConst*)\n", yylval.StringVal);
		} else if(lexeme == XIdf){
			printf("%s (*XIdf*)\n", yylval.StringVal);
		} else {
			int i;
			const char* r = NULL;
			for(i=0;i<NBKeywords;i++){
				if(lexeme == Keywords[i].token){
					r = Keywords[i].key;
					break;
				}
			}
			if(! r){
				for(i=0;i<NBSpecKeywords;i++){
					if(lexeme == SpecKeywords[i].token){
						r = SpecKeywords[i].key;
						break;
					}
				}
			}
			if(! r){
				printf("unknmown lexeme '%d'\n", lexeme);
				exit(1);
			} else {
				printf("%s\n", r);
			}
		}
	}
}
