(***********************************************************************
yacc2latex

Pascal Raymond, Verimag

http://www-verimag.imag.fr/~raymond/

Copyright 2012,2015 Pascal Raymond,
Centre National de la Recherche Scientifique

    This file is part of yacc2latex.

    yacc2latex is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    yacc2latex is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with yacc2latex.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************)

(* formal language grammar data type
   + associated functions
   very simple:
   - no "type" difference between terminal/non terminal
     they are all simple strings
     (basically a non-terminal is just a defined symbol)
*)

type symbol = string

(* single rule = sequence of symbols *)
type rule = symbol list

type pragma = string * string * string

type t

val dump : t -> unit

(*
   On ne cherche pas a savoir exactement qui sont les terminaux et
   les non termninaux, on se contente de distinguer :
   - les symboles definis (left) clairement des non terminaux
   - les litteraux (e.g. '+') qui sont clairement des terminaux

   On pourrait aussi considérer les pragma %token pour traquer
   les terminaux, mais ça ne sert pas à grand chose puisqu'on
   n'a pas la valeur concrete du token, et qu'il est
   impossible de la trouver automatiquement.
   Il y a le mécanisme des fichiers tokens pour ça, au niveau 
   de la traduction grammer2Ebnf

*) 
val def_symbols : t -> symbol list

val lit_symbols : t -> symbol list

val symbol_rules : t -> symbol -> rule list

val symbol_pragmas : t -> symbol -> pragma list

val get_symbol_pragma : t -> symbol -> (string * string) -> string

val of_yacc : Ylexer.t -> t
