(***********************************************************************
yacc2latex

Pascal Raymond, Verimag

http://www-verimag.imag.fr/~raymond/

Copyright 2012,2015 Pascal Raymond,
Centre National de la Recherche Scientifique

    This file is part of yacc2latex.

    yacc2latex is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    yacc2latex is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with yacc2latex.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************)
(*----------------------------------------------------------------------
 *	module : Verbose
 *	date :
 * 2016/02: add "os" (largely useless for the moment
 *          since necessarily  os=stderr !)
 * 2016 replaced "level" and "flag" by "l" anf "f"
 * 2010
------------------------------------------------------------------------
	description :

	Affichage verbeux avec appel "printf-like" :

Verbose.put "format" args...
----------------------------------------------------------------------*)


val on : unit -> unit
val off : unit -> unit
val set : int -> unit

val os : unit -> out_channel

(* FLAGS FOR FINE (DEBUG) VERBOSE
Typical use:

- in module Foo:
   *  add this global declaration ~ et the begining:
      let dbg = Global.get_dbg_flag "Toto"
   * then in the code use:
      Verbose.printf ~f:dbg ...
      Verbose.exe ~f:dbg ...

- integaration with Arg, in the main:
   * add a '-dbg' argument, e.g.:

      ... and
      arg_list =
      let dbgflags = Verbose.flag_list () in
      [
         ...
         ("-dbg", Arg.Symbol (dbgflags,
            (function s -> let x = Verbose.get_flag s in Verbose.set_flag x)),
            ": set debug flag"
         )
   
      ]
*)
type flag
val get_flag : string -> flag
val set_flag : flag -> unit
val flag_list : unit -> string list

(* val level : unit -> int *)

(* print/execute if EITHER:
   - level (dflt=1) is >= than the set level
   - flag is set
*)

(* 'why' system:
   - a string 'explaining' why the verbose is set,
     typically used to prefix the verbose messages
   - it is determined by the actual level, flag args
     * [dbg] for f=dbg
     * [v] for l=1, [v2] for v=2 etc.
   When using printf/print_string OUTSIDE exe context:
   - use xprintf/xprint_string instead to prefix with 'why'
   When using exe:
   - xprint... used within exe context inherit exe 'why'
   - if not suffucient, one can explicitely use the function:
     'Verbose.why ()' to insert the 'why string' 
   - N.B. outside the 'exe' context, why () does nothing 
   - WARNING: exe's CANNOT be nested (assertion violation)
*)
val xprintf : ?l:int -> ?f:flag -> ('a, unit, string, unit) format4 -> 'a
val xprint_string : ?l:int -> ?f:flag -> string -> unit 
val why : unit -> unit

(*
N.B. within 'exe' context, printf/print_string inherits l/f of the context
*)
val printf : ?l:int -> ?f:flag -> ('a, unit, string, unit) format4 -> 'a
val print_string : ?l:int -> ?f:flag -> string -> unit 
val exe : ?l:int -> ?f:flag -> (unit -> unit) -> unit
